
from pybricks.parameters import Axis, Direction, Port, Stop
from umath import atan2, degrees
from pybricks.pupdevices import Motor
from pybricks.hubs import InventorHub

# Initialize variables.
R = 0
left_angle = 0
right_angle = 0
l_speed = 100
r_speed = 0
LWRATIO = 0.9 # Length to width ratio of the rover wheel base

# Set up all devices.
inventor_hub = InventorHub(top_side=Axis.Z, front_side=Axis.X, broadcast_channel=1, observe_channels=[0])
left_steer = Motor(Port.A, Direction.CLOCKWISE)
right_steer = Motor(Port.B, Direction.CLOCKWISE)
left_drive = Motor(Port.C, Direction.COUNTERCLOCKWISE)
right_drive = Motor(Port.D, Direction.CLOCKWISE)

def deadzone(n, zone):
    # If a number n is within the jittery deadzone, return 0, else n.
    if -abs(zone) < n < abs(zone):
        return 0
    else:
        return n

def within90(angle):
    # Return an angle within +/- 90
    return (angle + 90) % 180 - 90

# The main program starts here.
while True:
    # Get l_speed and r_speed if there is data. Or make them 0.
    l_speed, r_speed = inventor_hub.ble.observe(0) or [0] * 2

    # Input values from tx are +/- 45. Times 20 they become +/- 900,
    # great for motor speeds.
    # Also compensate for deadzone input
    l_speed = deadzone(l_speed, 8) * 20
    r_speed = deadzone(r_speed, 8) * 20
    left_drive.run(l_speed)
    right_drive.run(r_speed)

    if (l_speed) != (r_speed): 
        # We're turning. 

        # Calculate turning radius from speed difference.
        # Unit of R = axle width. So R is actually R/2*h here.
        R = (r_speed + l_speed) / 2 / (r_speed - l_speed) 

        # Calculate wheel angles from turn radius.
        left_angle = degrees(atan2(-1 * LWRATIO, 2 * R - 1))
        right_angle = degrees(atan2(1 * LWRATIO, -(2 * R) - 1))
        
    else:
        # Driving straight and avoiding division by 0.
        left_angle = 0
        right_angle = 0

    # Steer and ensure the angle is within +/- 90
    # Also correct 15 degrees angle of the rocker bogie mechanism
    left_steer.track_target( within90(left_angle) + 15 )
    right_steer.track_target( within90(right_angle) - 15 )
